@@ -1,6 +1,7 @@
 '''
 Interact with data from the browser
 '''
+from datetime import datetime
 import io
 import os
 import re
@@ -228,14 +229,10 @@ def filter(url, args={}, meta={}, engine=None, ext=None, columns=None,
         # Get the full dataset. Then filter it
         data = gramex.cache.open(url, ext, transform=transform, **kwargs)
         return _filter_frame(data, meta=meta, controls=controls, args=args)
-    elif engine == 'plugin':
-        dbtype = url.split(':')[1]
-        if dbtype not in plugins:
-            raise ValueError(f'Unknown plugin:{dbtype}')
-        method = plugins[dbtype].get('filter', None)
-        if not callable(method):
-            raise ValueError(f'plugin:{dbtype}.filter not defined')
-        data = method(url=url[7:], controls=controls, args=args, query=query, **kwargs)
+    elif engine.startswith('plugin+'):
+        plugin = engine.split('+')[1]
+        method = plugins[plugin]['filter']
+        data = method(url=url, controls=controls, args=args, query=query, **kwargs)
         return _filter_frame(data, meta=meta, controls=controls, args=args)
     elif engine == 'sqlalchemy':
         table = kwargs.pop('table', None)
@@ -298,6 +295,11 @@ def delete(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
                                       args=args, source='delete', id=id)
         gramex.cache.save(data, url, ext, index=False, **kwargs)
         return len(data_filtered)
+    elif engine.startswith('plugin+'):
+        plugin = engine.split('+')[1]
+        method = plugins[plugin]['delete']
+        return method(url=url, meta=meta, controls=controls, args=args, id=id, table=table,
+                      columns=columns, ext=ext, query=query, queryfile=queryfile, **kwargs)
     elif engine == 'sqlalchemy':
         if table is None:
             raise ValueError('No table: specified')
@@ -338,6 +340,11 @@ def update(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
             data, meta=meta, controls=controls, args=args, source='update', id=id)
         gramex.cache.save(data, url, ext, index=False, **kwargs)
         return len(data_updated)
+    elif engine.startswith('plugin+'):
+        plugin = engine.split('+')[1]
+        method = plugins[plugin]['update']
+        return method(url=url, meta=meta, controls=controls, args=args, id=id, table=table,
+                      columns=columns, ext=ext, query=query, queryfile=queryfile, **kwargs)
     elif engine == 'sqlalchemy':
         if table is None:
             raise ValueError('No table: specified')
@@ -395,6 +402,10 @@ def insert(url, meta={}, args=None, engine=None, table=None, ext=None, id=None,
             data = data.append(rows, sort=False)
         gramex.cache.save(data, url, ext, index=False, **kwargs)
         return len(rows)
+    elif engine.startswith('plugin+'):
+        plugin = engine.split('+')[1]
+        method = plugins[plugin]['insert']
+        return method(url=url, rows=rows, meta=meta, args=args, table=table, **kwargs)
     elif engine == 'sqlalchemy':
         if table is None:
             raise ValueError('No table: specified')
@@ -448,7 +459,7 @@ def get_engine(url):
 
     - ``'dataframe'`` if url is a Pandas DataFrame
     - ``'sqlalchemy'`` if url is a sqlalchemy compatible URL
-    - ``'plugin'`` if it is `plugin:<name-of-plugin>`
+    - ``'plugin'`` if it is `<valid-plugin-name>://...`
     - ``protocol`` if url is of the form `protocol://...`
     - ``'dir'`` if it is not a URL but a valid directory
     - ``'file'`` if it is not a URL but a valid file
@@ -457,8 +468,9 @@ def get_engine(url):
     '''
     if isinstance(url, pd.DataFrame):
         return 'dataframe'
-    if url.startswith('plugin:'):
-        return 'plugin'
+    for plugin_name in plugins:
+        if url.startswith(f'{plugin_name}:'):
+            return f'plugin+{plugin_name}'
     try:
         url = sa.engine.url.make_url(url)
     except sa.exc.ArgumentError:
@@ -1359,44 +1371,40 @@ def _type_conversion(param_list, operations=None):
         raise ValueError('Value is not integer: %r' % param_list)
 
 
-# x>3&x>4&x>5 == {"$or": [{x: {$gt: 3}}, {x: {$gt: 4}}, {x: $gt: 5}]}
-# def _logical_conditions(args, meta_cols):
-#     conditions = []
-#     for key, vals in args.items():
-#         col, agg, op = _filter_col(key, meta_cols)
-#         convert = meta_cols[col].dtype.type
-#         conditions.append({col: {op: convert(val)} for val in vals})
-#     return {'$and': conditions}
+_mongodb_op_map = {
+    '<': '$lt',
+    '<~': '$lte',
+    '>': '$gt',
+    '>~': '$gte',
+    '': '$in',
+    '!': '$nin'
+}
 
 
-def _logical_conditions(args, meta_cols):
-    _op_mapping = {
-        '<': '$lt',
-        '<~': '$lte',
-        '>': '$gt',
-        '>~': '$gte',
-        '': '$in',
-        '!': '$nin'
-    }
-    _conditions = []
+def _filter_mongodb_col(col, op, vals, meta_cols):
+    if op in ['', '!']:
+        return {col: {_mongodb_op_map[op]: vals}}
+    elif op == '!~':
+        return {col: {"$not": {"$regex": '|'.join(vals), "$options": 'i'}}}
+    elif op == '~':
+        return {col: {"$regex": '|'.join(vals), "$options": 'i'}}
+    elif col and op in _mongodb_op_map.keys():
+        # TODO: Improve the numpy to Python type
+        convert = int if (meta_cols[col].dtype == pd.np.int64) else meta_cols[col].dtype.type
+        return {col: {_mongodb_op_map[op]: convert(val)} for val in vals}
+
+
+def _mongodb_query(args, meta_cols):
+    # Convert a query like x>=3&x>=4&x>=5 into
+    # {"$or": [{x: {$gt: 3}}, {x: {$gt: 4}}, {x: $gt: 5}]}
+    # TODO: ?_id= is not working
+    conditions = []
     for key, vals in args.items():
         col, agg, op = _filter_col(key, meta_cols)
-        if op in ['', '!']:
-            _conditions.append({col: {_op_mapping[op]: vals}})
-        elif op == '!~':
-            _conditions.append({col: {"$not": {"$regex": '|'.join(vals), "$options": 'i'}}})
-        elif op == '~':
-            _conditions.append({col: {"$regex": '|'.join(vals), "$options": 'i'}})
-        elif col and op in _op_mapping.keys():
-            # TODO: Improve the numpy to Python type
-            convert = int if (meta_cols[col].dtype == pd.np.int64) else meta_cols[col].dtype.type
-            _conditions.append({col: {_op_mapping[op]: convert(val)} for val in vals})
-
-    if len(_conditions) > 1:
-        return {'$and': _conditions}
-    if _conditions:
-        return _conditions[0]
-    return {}
+        if col:
+            conditions.append(_filter_mongodb_col(col, op, vals, meta_cols))
+        # TODO: add meta['ignored']
+    return {'$and': conditions} if len(conditions) > 1 else conditions[0] if conditions else {}
 
 
 def _controls_default(table, query=None, controls=None, meta_cols=None):
@@ -1427,47 +1435,245 @@ def _controls_default(table, query=None, controls=None, meta_cols=None):
     return cursor
 
 
-def _filter_mongodb(url, controls, args, database=None, collection=None, query=None, **kwargs):
-    '''
-        TODO: Document function and usage
-    '''
+def _mongodb_collection(url, database, collection, **kwargs):
     import pymongo
-    import bson
-    create_kwargs = {key: val for key, val in kwargs.items() if key in
-                     {'port', 'document_class', 'tz_aware', 'connect'}}
 
     # Create MongoClient
+    create_kwargs = {key: val for key, val in kwargs.items() if key in
+                     {'port', 'document_class', 'tz_aware', 'connect'}}
     client = create_engine(url, create=pymongo.MongoClient, **create_kwargs)
-
-    # TODO: check if database and collection exists
-
-    # client.list_databases()
-    # db.list_collection_names()
-
-    # Get Database
     db = client[database]
-    # Get Collection
-    table = db[collection]
+    return db[collection]
 
-    # TODO: Get metadata of requested document
-    meta_cols = pd.DataFrame(list(table.find().limit(100)))
 
-    if query:
-        _qbuilder = query
-    else:
-        _qbuilder = _logical_conditions(args, meta_cols)
-    cursor = _controls_default(table, query=_qbuilder, controls=controls, meta_cols=meta_cols)
+def _mongodb_json(obj):
+    '''Parse val in keys ending with . as JSON ({"key.": val}), but retain other keys'''
+    result = {}
+    for key, val in obj.items():
+        if key.endswith('.'):
+            result[key[:-1]] = json.loads(val)
+        else:
+            result[key] = val
+    return result
+
+
+def _filter_mongodb(url, controls, args, database=None, collection=None, query=None, **kwargs):
+    '''TODO: Document function and usage'''
+    table = _mongodb_collection(url, database, collection, **kwargs)
+    if query is None:
+        meta_cols = pd.DataFrame(list(table.find().limit(100)))
+        query = _mongodb_query(args, meta_cols)
+    cursor = _controls_default(table, query=query, controls=controls, meta_cols=meta_cols)
     data = pd.DataFrame(list(cursor))
-
-    # Convert Object IDs into strings
+    # Convert Object IDs into strings to allow JSON conversion
     if len(data) > 0:
+        import bson
         for col, val in data.iloc[0].iteritems():
             if type(val) in {bson.objectid.ObjectId}:
                 data[col] = data[col].map(str)
-
     return data
 
 
-plugins['mongodb'] = {
-    'filter': _filter_mongodb
+def _delete_mongodb(url, controls, args, meta=None, database=None, collection=None, query=None,
+                    **kwargs):
+    table = _mongodb_collection(url, database, collection, **kwargs)
+    meta_cols = pd.DataFrame(list(table.find().limit(100)))
+    query = _mongodb_query(args, meta_cols)
+    result = table.delete_many(query)
+    return result.deleted_count
+
+
+def _update_mongodb(url, controls, args, meta=None, database=None, collection=None, query=None,
+                    id=[], **kwargs):
+    table = _mongodb_collection(url, database, collection, **kwargs)
+    query = _mongodb_query(args, id)
+    values = {key: val[0] for key, val in args.items()}
+    result = table.update_many(query, {'$set': _mongodb_json(values)})
+    return result.modified_count
+
+
+def _insert_mongodb(url, rows, meta=None, database=None, collection=None, **kwargs):
+    table = _mongodb_collection(url, database, collection, **kwargs)
+    result = table.insert_many([_mongodb_json(row) for row in rows.to_dict(orient='records')])
+    meta['inserted'] = [{'id': str(id) for id in result.inserted_ids}]
+    return len(result.inserted_ids)
+
+
+def _get_influxdb_schema(client, bucket):
+    imports = 'import "influxdata/influxdb/schema"\n'
+    meas = client.query_api().query(
+        imports + f'schema.measurements(bucket: "{bucket}")'
+    )[0]
+    tags = client.query_api().query(imports + f'schema.tagKeys(bucket: "{bucket}")')[0]
+    tags = [r.get_value() for r in tags.records]
+    tags = [r for r in tags if not r.startswith("_")]
+    fields = client.query_api().query(
+        imports + f'schema.fieldKeys(bucket: "{bucket}")'
+    )[0]
+    return {
+        "_measurement": [r.get_value() for r in meas.records],
+        "_tags": tags,
+        "_fields": [r.get_value() for r in fields.records]
+    }
+
+
+_influxdb_op_map = {"<~": "<=", ">~": ">=", "!": "!=", "": "=="}
+
+
+def _influxdb_offset_limit(controls):
+    offset = controls.pop("_offset", ["-30d"])[0]
+    limit = controls.pop("_limit", [False])[0]
+    offset = f"start: {offset}"
+    if isinstance(limit, str) and not limit.isdigit():
+        offset += f", stop: {limit}"
+        limit = False
+
+    return offset, limit
+
+
+def _filter_influxdb(url, controls, args, org=None, bucket=None, query=None, **kwargs):
+    args.pop("bucket")
+    with _influxdb_client(url, org=org, **kwargs) as db:
+        schema = _get_influxdb_schema(db, bucket)
+        cols = schema["_fields"] + schema["_tags"] + schema["_measurement"]
+        q = db.query_api()
+        offset, limit = _influxdb_offset_limit(controls)
+        query = f'from(bucket: "{bucket}")|>range({offset})\n'
+
+        filters = []
+        wheres = []
+        to_drop = []
+        for col in controls.pop("_c", []):
+            if col.startswith("-"):
+                if col[1:] in schema["_fields"]:
+                    wheres.append(f'r._field != "{col[1:]}"')
+                else:
+                    to_drop.append(col[1:])
+            elif col in schema["_fields"]:
+                col, agg, op = _filter_col(col, cols)
+                op = _influxdb_op_map.get(op, op)
+                wheres.append(f'r._field {op} "{col}"')
+        if len(wheres):
+            wheres = " or ".join(wheres)
+            filters.append(f"|> filter(fn: (r) => {wheres})")
+        for key, vals in args.items():
+            col, agg, op = _filter_col(key, cols)
+            op = _influxdb_op_map.get(op, op)
+            if col in schema["_fields"]:
+                where = " or ".join([f'r._field == "{col}" and r._value {op} {v}' for v in vals])
+            else:
+                where = " or ".join([f'r["{col}"] {op} "{v}"' for v in vals])
+            filters.append(f"|> filter(fn: (r) => {where})")
+        query += "\n".join(filters)
+        if to_drop:
+            to_drop = ",".join([f'"{k}"' for k in to_drop])
+            query += f"\n|> drop(columns: [{to_drop}])"
+
+        if limit:
+            query += f"|> tail(n: {limit})\n"
+
+        app_log.debug("Running InfluxDB query: \n" + query)
+
+        df = q.query_data_frame(query)
+    return df.drop(["result", "table"], axis=1, errors="ignore")
+
+
+def _delete_influxdb(url, controls, args, org=None, bucket=None, **kwargs):
+    with _influxdb_client(url, org=org, **kwargs) as db:
+        schema = _get_influxdb_schema(db, bucket)
+        start = args.pop('_time>', ['0'])[0]
+        stop = args.pop('_time<', ['99999999999'])[0]
+        measurement = args.pop('_measurement', schema['_measurement'])[0]
+        predicate = f'_measurement="{measurement}"'
+        tags = [(k, v[0]) for k, v in args.items() if k in schema['_tags']]
+        if tags:
+            tag_predicate = " OR ".join([f'{k}="{v}"' for k, v in tags])
+            predicate += ' AND ' + f'({tag_predicate})'
+        db.delete_api().delete(start, stop, predicate, bucket, org)
+    # InfluxDB Python client doesn't return anything on DELETE, we return a mock number here.
+    return 0
+
+
+def _influxdb_client(url, token, org, debug=None, timeout=60_000, enable_gzip=False,
+                     default_tags=None, **kwargs):
+    from influxdb_client import InfluxDBClient
+
+    url = re.sub(r"^influxdb:", "", url)
+    return InfluxDBClient(
+        url, token, org=org, debug=debug, enable_gzip=enable_gzip, default_tags=default_tags,
+        timeout=timeout
+    )
+
+
+def _timestamp_df(df, index_col="_time"):
+    from tzlocal import get_localzone
+    now = datetime.now(get_localzone())
+    if index_col not in df:
+        df[index_col] = [now] * len(df)
+    else:
+        df[index_col] = pd.to_datetime(df[index_col], errors="coerce")
+        df[index_col].fillna(value=now, inplace=True)
+    return df.set_index(index_col)
+
+
+def _get_ts_points(df, measurement, tags):
+    from influxdb_client import Point
+
+    points = []
+    tags = df[tags]
+    fields = df.drop(tags, axis=1)
+    for (t, field), (_, tag) in zip(fields.astype(float).iterrows(), tags.iterrows()):
+        p = Point(measurement).time(t)
+        [p.tag(t, tval) for t, tval in tag.to_dict().items()]
+        [p.field(f, fval) for f, fval in field.to_dict().items()]
+        points.append(p)
+    return points
+
+
+def _insert_influxdb(url, rows, meta, args, bucket, **kwargs):
+    measurement = rows.pop("measurement").unique()[0]
+    tags = (
+        rows.pop("tags").dropna().drop_duplicates().tolist() if "tags" in rows else []
+    )
+    # Ensure that the index is timestamped
+    rows = _timestamp_df(rows)
+
+    # Ensure that all columns except tags are floats
+    field_columns = [c for c in rows if c not in tags]
+    rows[field_columns] = rows[field_columns].astype(float)
+    from influxdb_client.client.write_api import ASYNCHRONOUS, WriteOptions
+
+    with _influxdb_client(url, **kwargs) as db:
+        with db.write_api(
+            write_options=WriteOptions(
+                ASYNCHRONOUS, batch_size=50_000, flush_interval=10_000
+            )
+        ) as client:
+            client.write(
+                bucket=bucket,
+                org=db.org,
+                record=rows,
+                data_frame_measurement_name=measurement,
+                data_frame_tag_columns=tags,
+            )
+    meta["inserted"] = [{"id": ix} for ix, _ in rows.iterrows()]
+    return len(rows)
+
+
+# add test case for inserting nested value ?parent.={child:value}
+#   curl --globoff -I -X POST 'http://127.0.0.1:9988/?x.={"2":3}&y.={"true":true}&Name=abcd'
+# add test case for updating nested value ?parent.child.={key:value}
+#   curl --globoff -I -X PUT 'http://127.0.0.1:9988/?x.2=4&y.true.=[2,3]&Name=abcd'
+# add test case for nested document query ?parent.child=value
+plugins["mongodb"] = {
+    "filter": _filter_mongodb,
+    "delete": _delete_mongodb,
+    "insert": _insert_mongodb,
+    "update": _update_mongodb,
+}
+plugins["influxdb"] = {
+    "filter": _filter_influxdb,
+    "delete": _delete_influxdb,
+    "insert": _insert_influxdb,
+    "update": _insert_influxdb,
 }
