@@ -14,6 +14,7 @@ from gramex.install import _mkdir, safe_rmtree
 from gramex import cache
 import joblib
 import pandas as pd
+from sklearn.base import TransformerMixin
 from sklearn.compose import ColumnTransformer
 from sklearn.pipeline import Pipeline
 from sklearn.preprocessing import OneHotEncoder, StandardScaler
@@ -31,6 +32,8 @@ MLCLASS_MODULES = [
     'sklearn.neighbors',
     'sklearn.neural_network',
     'sklearn.naive_bayes',
+    'sklearn.decomposition',
+    'gramex.ml'
 ]
 TRANSFORMS = {
     'include': [],
@@ -47,7 +50,7 @@ DEFAULT_TEMPLATE = op.join(op.dirname(__file__), '..', 'apps', 'mlhandler', 'tem
 search_modelclass = lambda x: locate(x, MLCLASS_MODULES)  # NOQA: E731
 
 
-def _fit(model, x, y, path=None, name=None):
+def _fit(model, x, y=None, path=None, name=None):
     app_log.info('Starting training...')
     try:
         getattr(model, 'partial_fit', model.fit)(x, y)
@@ -114,8 +117,12 @@ class MLHandler(FormHandler):
             cls.model = cls._assemble_pipeline(data, mclass=mclass, params=params)
 
             # train the model
-            target = data[target_col]
-            train = data[[c for c in data if c != target_col]]
+            if issubclass(search_modelclass(mclass), TransformerMixin):
+                target = None
+                train = data
+            else:
+                target = data[target_col]
+                train = data.drop([target_col], axis=1)
             gramex.service.threadpool.submit(
                 _fit, cls.model, train, target, cls.model_path, cls.name)
         cls.config_store.flush()
@@ -277,7 +284,10 @@ class MLHandler(FormHandler):
             # Set data in the same order as the transformer requests
             try:
                 data = data[self.model.named_steps['transform']._feature_names_in]
-                data[self.get_opt('target_col', '_prediction')] = self.model.predict(data)
+                if isinstance(self.model[-1], TransformerMixin):
+                    data = self.model.transform(data)
+                else:
+                    data[self.get_opt('target_col', '_prediction')] = self.model.predict(data)
             except Exception as exc:
                 app_log.exception(exc)
             return data
@@ -304,7 +314,15 @@ class MLHandler(FormHandler):
                 'opts': self.config_store.load('transform'),
                 'params': self.config_store.load('model')
             }
-            self.write(json.dumps(params, indent=2))
+            try:
+                model = cache.open(self.model_path, joblib.load)
+                attrs = {
+                    k: v for k, v in vars(model[-1]).items() if re.search(r'[^_]+_$', k)
+                }
+            except FileNotFoundError:
+                attrs = {}
+            params['attrs'] = attrs
+            self.write(json.dumps(params, indent=2, cls=CustomJSONEncoder))
         elif '_cache' in self.args:
             self.write(self.load_data().to_json(orient='records'))
         else:
@@ -348,11 +366,19 @@ class MLHandler(FormHandler):
         data = self._parse_data(False) if data is None else data
         data = self._filtercols(data)
         data = self._filterrows(data)
-        target = data[target_col]
-        train = data[[c for c in data if c != target_col]]
         self.model = self._assemble_pipeline(data, force=True)
-        _fit(self.model, train, target, self.model_path)
-        return {'score': self.model.score(train, target)}
+        if not isinstance(self.model[-1], TransformerMixin):
+            target = data[target_col]
+            train = data[[c for c in data if c != target_col]]
+            _fit(self.model, train, target, self.model_path)
+            result = {'score': self.model.score(train, target)}
+        else:
+            _fit(self.model, data, path=self.model_path)
+            # Note: Fitted sklearn estimators store their parameters
+            # in attributes whose names end in an underscore. E.g. in the case of PCA,
+            # attributes are named `explained_variance_`. The `_train` action returns them.
+            result = {k: v for k, v in vars(self.model[-1]).items() if re.search(r'[^_]+_$', k)}
+        return result
 
     def _retrain(self):
         return self._train(self.load_data())
